from unittest.mock import patch

import pytest
from infoblox_client import objects
from orchestrator.types import SubscriptionLifecycle

from gso.products.product_blocks.router import RouterRole
from gso.products.product_types.router import Router
from gso.utils.shared_enums import Vendor
from test.services.conftest import MockedKentikClient
from test.workflows import (
    assert_complete,
    assert_lso_success,
    extract_state,
    run_workflow,
)


@pytest.mark.parametrize("router_state", [SubscriptionLifecycle.PROVISIONING, SubscriptionLifecycle.ACTIVE])
@pytest.mark.parametrize("router_role", [RouterRole.P, RouterRole.PE])
@pytest.mark.workflow()
@patch("gso.services.infoblox.find_host_by_fqdn")
@patch("gso.services.lso_client._send_request")
@patch("gso.services.netbox_client.NetboxClient.get_device_by_name")
@patch("gso.services.librenms_client.LibreNMSClient.validate_device")
@patch("gso.services.kentik_client.KentikClient")
def test_validate_nokia_router_success(
    mock_kentik_client,
    mock_validate_librenms_device,
    mock_get_device_by_name,
    mock_execute_playbook,
    mock_find_host_by_fqdn,
    router_subscription_factory,
    faker,
    router_state,
    router_role,
):
    mock_validate_librenms_device.return_value = None
    router_subscription_factory(router_role=RouterRole.P)
    router_subscription_factory(router_role=RouterRole.PE)
    mock_kentik_client.return_value = MockedKentikClient
    #  Run workflow
    subscription_id = router_subscription_factory(status=router_state, router_role=router_role)
    mock_fqdn = Router.from_subscription(subscription_id).router.router_fqdn
    mock_v4 = faker.ipv4()
    mock_find_host_by_fqdn.return_value = objects.HostRecord(
        connector=None,
        aliases=[mock_fqdn],
        comment=subscription_id,
        ipv4addrs=[
            objects.IPv4(
                ipv4addr=str(mock_v4),
                configure_for_dhcp=False,
                mac="00:00:00:00:00:00",
                ip=str(mock_v4),
                host=f"lo0.{mock_fqdn}",
            ),
        ],
        name=mock_fqdn,
    )

    initial_router_data = [{"subscription_id": subscription_id}]
    result, process_stat, step_log = run_workflow("validate_router", initial_router_data)

    state = extract_state(result)
    subscription_id = state["subscription_id"]

    lso_execution_count = 2 if router_role == RouterRole.P else 3

    for _ in range(lso_execution_count):
        result, step_log = assert_lso_success(result, process_stat, step_log)

    assert_complete(result)

    state = extract_state(result)
    subscription = Router.from_subscription(subscription_id)

    assert subscription.status == router_state
    assert mock_execute_playbook.call_count == lso_execution_count
    assert mock_find_host_by_fqdn.call_count == 1
    assert mock_get_device_by_name.call_count == 1
    assert mock_validate_librenms_device.call_count == 1
    assert "ipam_warning" not in state


@pytest.mark.workflow()
def test_validate_juniper_router_success(router_subscription_factory):
    #  Run workflow
    subscription_id = router_subscription_factory(vendor=Vendor.JUNIPER)

    initial_router_data = [{"subscription_id": subscription_id}]
    result, _, _ = run_workflow("validate_router", initial_router_data)

    assert_complete(result)
    state = extract_state(result)
    subscription = Router.from_subscription(subscription_id)

    assert subscription.status == "active"
    assert "ipam_warning" not in state
